@@ -36,6 +36,7 @@ static bool ui_draw(struct ui *ui, const struct pl_swapchain_frame *frame) { ret
 #include <libplacebo/utils/frame_queue.h>
 
 #define MAX_FRAME_PASSES 256
+#define MAX_BLEND_PASSES 8
 #define MAX_BLEND_FRAMES 8
 
 struct pass_info {
@@ -69,6 +70,7 @@ struct plplay {
     struct pl_peak_detect_params peak_detect_params;
     struct pl_color_map_params color_map_params;
     struct pl_dither_params dither_params;
+    struct pl_deinterlace_params deinterlace_params;
     struct pl_icc_params icc_params;
     struct pl_cone_params cone_params;
     struct pl_color_space target_color;
@@ -87,9 +89,10 @@ struct plplay {
     size_t shader_size;
 
     // pass metadata
-    struct pass_info blend_info[MAX_BLEND_FRAMES];
+    struct pass_info blend_info[MAX_BLEND_FRAMES][MAX_BLEND_PASSES];
     struct pass_info frame_info[MAX_FRAME_PASSES];
     int num_frame_passes;
+    int num_blend_passes[MAX_BLEND_FRAMES];
 };
 
 static void uninit(struct plplay *p)
@@ -217,6 +220,7 @@ static bool init_codec(struct plplay *p)
 
         const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(hwcfg->pix_fmt);
         printf("Using hardware frame format: %s\n", desc->name);
+        p->codec->extra_hw_frames = 4;
         break;
     }
 
@@ -287,6 +291,7 @@ static void *decode_loop(void *arg)
     if (!frame || !packet)
         goto done;
 
+    float frame_duration = av_q2d(av_inv_q(p->stream->avg_frame_rate));
     double first_pts = 0.0, base_pts = 0.0, last_pts = 0.0;
     uint64_t num_frames = 0;
 
@@ -325,10 +330,16 @@ static void *decode_loop(void *arg)
             frame->opaque = p;
             pl_queue_push_block(p->queue, UINT64_MAX, &(struct pl_source_frame) {
                 .pts = last_pts - first_pts + base_pts,
+                .duration = frame_duration,
                 .map = map_frame,
                 .unmap = unmap_frame,
                 .discard = discard_frame,
                 .frame_data = frame,
+
+                // allow soft-disabling deinterlacing at the source frame level
+                .first_field = p->params.deinterlace_params
+                                    ? pl_field_from_avframe(frame)
+                                    : PL_FIELD_NONE,
             });
             frame = av_frame_alloc();
         }
@@ -422,8 +433,6 @@ static bool render_frame(struct plplay *p, const struct pl_swapchain_frame *fram
 static bool render_loop(struct plplay *p)
 {
     struct pl_queue_params qparams = {
-        .radius = pl_frame_mix_radius(&p->params),
-        .frame_duration = av_q2d(av_inv_q(p->stream->avg_frame_rate)),
         .interpolation_threshold = 0.01,
         .timeout = UINT64_MAX,
     };
@@ -487,6 +496,7 @@ retry:
         }
         ts_prev = ts;
 
+        qparams.radius = pl_frame_mix_radius(&p->params);
         qparams.timeout = 50000000; // 50 ms
         qparams.pts = pts;
 
@@ -532,9 +542,10 @@ static void info_callback(void *priv, const struct pl_render_info *info)
         break;
 
     case PL_RENDER_STAGE_BLEND:
-        if (info->index >= MAX_BLEND_FRAMES)
+        if (info->index >= MAX_BLEND_PASSES || info->count >= MAX_BLEND_FRAMES)
             return;
-        pass = &p->blend_info[info->index];
+        p->num_blend_passes[info->count] = info->index + 1;
+        pass = &p->blend_info[info->count][info->index];
         break;
 
     case PL_RENDER_STAGE_COUNT: abort();
@@ -574,6 +585,7 @@ int main(int argc, char **argv)
         .dither_params = pl_dither_default_params,
         .icc_params = pl_icc_default_params,
         .cone_params = pl_vision_normal,
+        .deinterlace_params = pl_deinterlace_default_params,
         .target_override = true,
     };
 
@@ -585,6 +597,7 @@ int main(int argc, char **argv)
     DEFAULT_PARAMS(sigmoid_params);
     DEFAULT_PARAMS(peak_detect_params);
     DEFAULT_PARAMS(dither_params);
+    DEFAULT_PARAMS(deinterlace_params);
     state.params.color_adjustment = &state.color_adjustment;
     state.params.color_map_params = &state.color_map_params;
     state.params.cone_params = &state.cone_params;
@@ -881,6 +894,35 @@ static void update_settings(struct plplay *p)
             nk_tree_pop(nk);
         }
 
+        if (nk_tree_push(nk, NK_TREE_NODE, "Deinterlacing", NK_MINIMIZED)) {
+            struct pl_deinterlace_params *dpar = &p->deinterlace_params;
+            nk_layout_row_dynamic(nk, 24, 2);
+            par->deinterlace_params = nk_check_label(nk, "Enable", par->deinterlace_params) ? dpar : NULL;
+            if (nk_button_label(nk, "Reset settings"))
+                *dpar = pl_deinterlace_default_params;
+
+            static const char *deint_algos[PL_DEINTERLACE_ALGORITHM_COUNT] = {
+                [PL_DEINTERLACE_WEAVE]  = "Field weaving (no-op)",
+                [PL_DEINTERLACE_BOB]    = "Naive bob (line doubling)",
+                [PL_DEINTERLACE_YADIF]  = "Yadif (\"yet another deinterlacing filter\")",
+            };
+
+            nk_label(nk, "Deinterlacing algorithm", NK_TEXT_LEFT);
+            dpar->algo = nk_combo(nk, deint_algos, PL_DEINTERLACE_ALGORITHM_COUNT,
+                                  dpar->algo, 16, nk_vec2(nk_widget_width(nk), 300));
+
+            switch (dpar->algo) {
+            case PL_DEINTERLACE_WEAVE:
+            case PL_DEINTERLACE_BOB:
+                break;
+            case PL_DEINTERLACE_YADIF:
+                nk_checkbox_label(nk, "Skip spatial check", &dpar->skip_spatial_check);
+                break;
+            default: abort();
+            }
+            nk_tree_pop(nk);
+        }
+
         if (nk_tree_push(nk, NK_TREE_NODE, "Debanding", NK_MINIMIZED)) {
             struct pl_deband_params *dpar = &p->deband_params;
             nk_layout_row_dynamic(nk, 24, 2);
@@ -1105,6 +1147,21 @@ static void update_settings(struct plplay *p)
 
             nk_checkbox_label(nk, "Temporal dithering", &dpar->temporal);
 
+            nk_layout_row_dynamic(nk, 24, 2);
+            nk_label(nk, "Error diffusion:", NK_TEXT_LEFT);
+            const char *name = par->error_diffusion ? par->error_diffusion->description : "(None)";
+            if (nk_combo_begin_label(nk, name, nk_vec2(nk_widget_width(nk), 500))) {
+                nk_layout_row_dynamic(nk, 16, 1);
+                if (nk_combo_item_label(nk, "(None)", NK_TEXT_LEFT))
+                    par->error_diffusion = NULL;
+                for (int i = 0; i < pl_num_error_diffusion_kernels; i++) {
+                    const struct pl_error_diffusion_kernel *k = pl_error_diffusion_kernels[i];
+                    if (nk_combo_item_label(nk, k->description, NK_TEXT_LEFT))
+                        par->error_diffusion = k;
+                }
+                nk_combo_end(nk);
+            }
+
             nk_tree_pop(nk);
         }
 
@@ -1259,9 +1316,11 @@ static void update_settings(struct plplay *p)
             }
 
             if (p->target_icc.len) {
-                nk_layout_row(nk, NK_DYNAMIC, 24, 2, (float[]){ 0.7, 0.3 });
+                nk_layout_row_dynamic(nk, 24, 1);
                 nk_labelf(nk, NK_TEXT_LEFT, "Loaded: %s",
                           p->target_icc_name ? p->target_icc_name : "(unknown)");
+                nk_layout_row_dynamic(nk, 24, 2);
+                nk_checkbox_label(nk, "Force BPC", &iccpar->force_bpc);
                 reset_icc |= nk_button_label(nk, "Reset ICC");
             }
 
@@ -1282,7 +1341,7 @@ static void update_settings(struct plplay *p)
 
             if (reset_levels) {
                 tcol->hdr = (struct pl_hdr_metadata) {0};
-                *iccpar = pl_icc_default_params;
+                iccpar->max_luma = 0;
             }
 
             nk_tree_pop(nk);
@@ -1363,6 +1422,7 @@ static void update_settings(struct plplay *p)
             nk_checkbox_label(nk, "Disable linear scaling", &par->disable_linear_scaling);
             nk_checkbox_label(nk, "Disable built-in scalers", &par->disable_builtin_scalers);
             nk_checkbox_label(nk, "Force-enable dither", &par->force_dither);
+            nk_checkbox_label(nk, "Disable gamma-aware dither", &par->disable_dither_gamma_correction);
             nk_checkbox_label(nk, "Disable FBOs / advanced rendering", &par->disable_fbos);
             nk_checkbox_label(nk, "Force low-bit depth FBOs", &par->force_low_bit_depth_fbos);
             nk_checkbox_label(nk, "Disable constant hard-coding", &par->dynamic_constants);
@@ -1408,27 +1468,29 @@ static void update_settings(struct plplay *p)
 
                 nk_layout_row_dynamic(nk, 26, 1);
                 nk_label(nk, "Output blending:", NK_TEXT_LEFT);
-                for (int i = 0; i < MAX_BLEND_FRAMES; i++) {
-                    struct pass_info *info = &p->blend_info[i];
-                    if (!info->name)
-                        continue;
+                for (int j = 0; j < MAX_BLEND_FRAMES; j++) {
+                    for (int i = 0; i < p->num_blend_passes[j]; i++) {
+                        struct pass_info *info = &p->blend_info[j][i];
+                        if (!info->name)
+                            continue;
 
-                    nk_layout_row_dynamic(nk, 24, 1);
-                    nk_labelf(nk, NK_TEXT_LEFT,
-                              "- (%d frame%s) %s: %.3f / %.3f / %.3f ms",
-                              i, i > 1 ? "s" : "", info->name,
-                              info->pass.last / 1e6,
-                              info->pass.average / 1e6,
-                              info->pass.peak / 1e6);
+                        nk_layout_row_dynamic(nk, 24, 1);
+                        nk_labelf(nk, NK_TEXT_LEFT,
+                                  "- (%d frame%s) %s: %.3f / %.3f / %.3f ms",
+                                  j, j == 1 ? "" : "s", info->name,
+                                  info->pass.last / 1e6,
+                                  info->pass.average / 1e6,
+                                  info->pass.peak / 1e6);
 
-                    nk_layout_row_dynamic(nk, 32, 1);
-                    if (nk_chart_begin(nk, NK_CHART_LINES,
-                                       info->pass.num_samples,
-                                       0.0f, info->pass.peak))
-                    {
-                        for (int k = 0; k < info->pass.num_samples; k++)
-                            nk_chart_push(nk, info->pass.samples[k]);
-                        nk_chart_end(nk);
+                        nk_layout_row_dynamic(nk, 32, 1);
+                        if (nk_chart_begin(nk, NK_CHART_LINES,
+                                           info->pass.num_samples,
+                                           0.0f, info->pass.peak))
+                        {
+                            for (int k = 0; k < info->pass.num_samples; k++)
+                                nk_chart_push(nk, info->pass.samples[k]);
+                            nk_chart_end(nk);
+                        }
                     }
                 }
 
