@@ -406,9 +406,7 @@ void pl_color_space_infer(struct pl_color_space *space)
         space->sig_floor = 0;
     }
 
-    if (space->hdr.max_luma < space->hdr.min_luma) // sanity
-        space->hdr.max_luma = space->hdr.min_luma = 0;
-
+reinfer_peaks:
     if (space->hdr.max_luma < 1 || space->hdr.max_luma > 10000) {
         space->hdr.max_luma = pl_color_transfer_nominal_peak(space->transfer)
                               * PL_COLOR_SDR_WHITE;
@@ -421,12 +419,28 @@ void pl_color_space_infer(struct pl_color_space *space)
 
     if (space->hdr.min_luma <= 0 || space->hdr.min_luma > 100) {
         if (pl_color_transfer_is_hdr(space->transfer)) {
-            space->hdr.min_luma = 0.0050f; // Typical HDR black
+            // Use a slightly nonzero black level, for the following reasons:
+            // - 0 may be seen as 'missing/undefined' in HDR10 metadata structs
+            //
+            // - we need to clamp to 1e-7 to avoid issues with some tone
+            //   mapping functions anyway, so doing it here also advertises
+            //   this fact downstream
+            //
+            // - true infinite contrast does not exist in reality, even 1e-7
+            //   is extremely generous considering typical viewing environments
+            //   are not absolutely devoid of stray ambient light
+            space->hdr.min_luma = 1e-7;
         } else {
             space->hdr.min_luma = space->hdr.max_luma / 1000; // Typical SDR contrast
         }
     }
 
+    pl_assert(space->hdr.min_luma && space->hdr.max_luma);
+    if (space->hdr.max_luma < space->hdr.min_luma) { // sanity
+        space->hdr.max_luma = space->hdr.min_luma = 0;
+        goto reinfer_peaks;
+    }
+
     if (space->sig_scale && !pl_color_transfer_is_hdr(space->transfer)) {
         space->hdr.max_luma *= space->sig_scale;
         space->hdr.min_luma *= space->sig_scale;
@@ -440,27 +454,26 @@ void pl_color_space_infer(struct pl_color_space *space)
     pl_raw_primaries_merge(&space->hdr.prim, pl_raw_primaries_get(space->primaries));
 }
 
-void pl_color_space_infer_ref(struct pl_color_space *space,
-                              const struct pl_color_space *refp)
+static void infer_both_ref(struct pl_color_space *space,
+                           struct pl_color_space *ref)
 {
-    struct pl_color_space ref = *refp;
-    pl_color_space_infer(&ref);
+    pl_color_space_infer(ref);
 
     if (!space->primaries) {
-        if (pl_color_primaries_is_wide_gamut(ref.primaries)) {
+        if (pl_color_primaries_is_wide_gamut(ref->primaries)) {
             space->primaries = PL_COLOR_PRIM_BT_709;
         } else {
-            space->primaries = ref.primaries;
+            space->primaries = ref->primaries;
         }
     }
 
     if (!space->transfer) {
-        if (pl_color_transfer_is_hdr(ref.transfer)) {
+        if (pl_color_transfer_is_hdr(ref->transfer)) {
             space->transfer = PL_COLOR_TRC_BT_1886;
-        } else if (ref.transfer == PL_COLOR_TRC_LINEAR) {
+        } else if (ref->transfer == PL_COLOR_TRC_LINEAR) {
             space->transfer = PL_COLOR_TRC_GAMMA22;
         } else {
-            space->transfer = ref.transfer;
+            space->transfer = ref->transfer;
         }
     }
 
@@ -468,6 +481,42 @@ void pl_color_space_infer_ref(struct pl_color_space *space,
     pl_color_space_infer(space);
 }
 
+void pl_color_space_infer_ref(struct pl_color_space *space,
+                              const struct pl_color_space *refp)
+{
+    // Make a copy of `refp` to infer missing values first
+    struct pl_color_space ref = *refp;
+    infer_both_ref(space, &ref);
+}
+
+void pl_color_space_infer_map(struct pl_color_space *src,
+                              struct pl_color_space *dst)
+{
+    bool unknown_contrast = !src->hdr.min_luma;
+
+    // PQ is always scaled down to absolute black, regardless of what the
+    // metadata says. So treat it as such. We do this here, rather than in e.g.
+    // pl_color_space_infer, because the latter is also used to sanitize values
+    // in cases where we don't want to override mastering metadata.
+    // Additionally, doing it here allows e.g. correct propagation of black
+    // levels for BT.1886 -> PQ expansion in the step further below.
+    if (src->transfer == PL_COLOR_TRC_PQ)
+        src->hdr.min_luma = 0;
+    if (dst->transfer == PL_COLOR_TRC_PQ)
+        dst->hdr.min_luma = 0;
+
+    infer_both_ref(dst, src);
+
+    // If the src has an unspecified SDR-like gamma, default it to match the
+    // dst colorspace contrast. This does not matter in most cases, but ensures
+    // that BT.1886 is tuned to the appropriate black point by default.
+    bool dynamic_contrast = pl_color_space_is_black_scaled(src) ||
+                            src->transfer == PL_COLOR_TRC_BT_1886;
+
+    if (unknown_contrast && dynamic_contrast)
+        src->hdr.min_luma = dst->hdr.min_luma;
+}
+
 const struct pl_color_adjustment pl_color_adjustment_neutral = {
     .brightness     = 0.0,
     .contrast       = 1.0,
