@@ -29,8 +29,10 @@ struct icc_priv {
     cmsContext cms;
     cmsHPROFILE profile;
     cmsHPROFILE approx; // approximation profile
+    float a, b, scale; // approxmation tone curve parameters and scaling
     cmsCIEXYZ black;
     float gamma_stddev;
+    uint64_t lut_sig;
 };
 
 static void error_callback(cmsContext cms, cmsUInt32Number code,
@@ -53,8 +55,8 @@ void pl_icc_close(pl_icc_object *picc)
     pl_free_ptr((void **) picc);
 }
 
-static bool detect_csp(pl_icc_object icc, struct pl_raw_primaries *prim,
-                       float *gamma)
+static bool detect_csp(pl_log pllog, pl_icc_object icc,
+                       struct pl_raw_primaries *prim, float *gamma)
 {
     struct icc_priv *p = PL_PRIV(icc);
     cmsHTRANSFORM tf;
@@ -75,11 +77,23 @@ static bool detect_csp(pl_icc_object icc, struct pl_raw_primaries *prim,
     if (!tf)
         return false;
 
+    enum {
+        RED,
+        GREEN,
+        BLUE,
+        WHITE,
+        BLACK,
+        GRAY,
+        RAMP,
+    };
+
     static const uint8_t test[][3] = {
-        { 0xFF,    0,    0 }, // red
-        {    0, 0xFF,    0 }, // green
-        {    0,    0, 0xFF }, // blue
-        { 0xFF, 0xFF, 0xFF }, // white
+        [RED]   = { 0xFF,    0,    0 },
+        [GREEN] = {    0, 0xFF,    0 },
+        [BLUE]  = {    0,    0, 0xFF },
+        [WHITE] = { 0xFF, 0xFF, 0xFF },
+        [BLACK] = { 0x00, 0x00, 0x00 },
+        [GRAY]  = { 0x80, 0x80, 0x80 },
 
         // Grayscale ramp (excluding endpoints)
 #define V(d) { d, d, d }
@@ -123,26 +137,42 @@ static bool detect_csp(pl_icc_object icc, struct pl_raw_primaries *prim,
     cmsDeleteTransform(tf);
 
     // Read primaries from transformed RGBW values
-    prim->red   = pl_cie_from_XYZ(dst[0].X, dst[0].Y, dst[0].Z);
-    prim->green = pl_cie_from_XYZ(dst[1].X, dst[1].Y, dst[1].Z);
-    prim->blue  = pl_cie_from_XYZ(dst[2].X, dst[2].Y, dst[2].Z);
-    prim->white = pl_cie_from_XYZ(dst[3].X, dst[3].Y, dst[3].Z);
+    prim->red   = pl_cie_from_XYZ(dst[RED].X, dst[RED].Y, dst[RED].Z);
+    prim->green = pl_cie_from_XYZ(dst[GREEN].X, dst[GREEN].Y, dst[GREEN].Z);
+    prim->blue  = pl_cie_from_XYZ(dst[BLUE].X, dst[BLUE].Y, dst[BLUE].Z);
+    prim->white = pl_cie_from_XYZ(dst[WHITE].X, dst[WHITE].Y, dst[WHITE].Z);
+
+    // Rough estimate of overall gamma and starting point for curve black point
+    const float y_approx = log(dst[GRAY].Y) / log(0.5);
+    const float kb = fmaxf(dst[BLACK].Y, 0.0f);
+    float b = powf(kb, 1 / y_approx);
 
     // Estimate mean and stddev of gamma (Welford's method)
     float M = 0.0, S = 0.0;
     int k = 1;
-    for (int i = 4; i < PL_ARRAY_SIZE(dst); i++) { // exclude RGBW cube
+    for (int i = RAMP; i < PL_ARRAY_SIZE(dst); i++) { // exclude primaries
         if (dst[i].Y <= 0 || dst[i].Y >= 1)
             continue;
-        float y = log(dst[i].Y) / log(test[i][0] / 255.0);
+        float src = (1 - b) * (test[i][0] / 255.0) + b;
+        float y = log(dst[i].Y) / log(src);
         float tmpM = M;
         M += (y - tmpM) / k;
         S += (y - tmpM) * (y - M);
         k++;
+
+        // Update estimate of black point according to current gamma estimate
+        b = powf(kb, 1 / M);
     }
     S = sqrt(S / (k - 1));
-    if (S > 0.5)
-        return false; // too big deviation, probably something broken??
+    if (S > 0.5) {
+        pl_warn(pllog, "Detected profile gamma (%.3f) very far from pure power "
+                "response (stddev=%.1f), suspected unusual or broken profile. "
+                "Ignoring, but results may be poor.", M, S);
+    } else if (!(M > 0)) {
+        pl_err(pllog, "Arithmetic error in ICC profile gamma estimation? "
+               "Please open an issue");
+        return false;
+    }
 
     *gamma = M;
     p->gamma_stddev = S;
@@ -154,7 +184,13 @@ static bool detect_contrast(pl_icc_object icc, struct pl_hdr_metadata *hdr,
 {
     struct icc_priv *p = PL_PRIV(icc);
     cmsCIEXYZ *white = cmsReadTag(p->profile, cmsSigLuminanceTag);
-    if (!cmsDetectDestinationBlackPoint(&p->black, p->profile, icc->params.intent, 0))
+    enum pl_rendering_intent intent = icc->params.intent;
+    /* LittleCMS refuses to detect an intent in absolute colorimetric intent,
+     * so fall back to relative colorimetric since we only care about the
+     * brightness value here */
+    if (intent == PL_INTENT_ABSOLUTE_COLORIMETRIC)
+        intent = PL_INTENT_RELATIVE_COLORIMETRIC;
+    if (!cmsDetectDestinationBlackPoint(&p->black, p->profile, intent, 0))
         return false;
 
     if (max_luma <= 0)
@@ -166,7 +202,7 @@ static bool detect_contrast(pl_icc_object icc, struct pl_hdr_metadata *hdr,
     return true;
 }
 
-static void infer_clut_size(pl_log log, struct pl_icc_object *icc)
+static void infer_clut_size(pl_log log, struct pl_icc_object_t *icc)
 {
     struct icc_priv *p = PL_PRIV(icc);
     struct pl_icc_params *params = &icc->params;
@@ -285,7 +321,7 @@ pl_icc_object pl_icc_open(pl_log log, const struct pl_icc_profile *profile,
     if (!profile->data)
         return NULL;
 
-    struct pl_icc_object *icc = pl_zalloc_obj(NULL, icc, struct icc_priv);
+    struct pl_icc_object_t *icc = pl_zalloc_obj(NULL, icc, struct icc_priv);
     struct icc_priv *p = PL_PRIV(icc);
     struct pl_icc_params *params = &icc->params;
     *params = pparams ? *pparams : pl_icc_default_params;
@@ -313,7 +349,7 @@ pl_icc_object pl_icc_open(pl_log log, const struct pl_icc_profile *profile,
         params->intent = cmsGetHeaderRenderingIntent(p->profile);
 
     struct pl_raw_primaries *out_prim = &icc->csp.hdr.prim;
-    if (!detect_csp(icc, out_prim, &icc->gamma))
+    if (!detect_csp(log, icc, out_prim, &icc->gamma))
         goto error;
     if (!detect_contrast(icc, &icc->csp.hdr, params->max_luma))
         goto error;
@@ -342,8 +378,18 @@ pl_icc_object pl_icc_open(pl_log log, const struct pl_icc_profile *profile,
         goto error;
     }
 
-    // Create approximation profile
-    cmsToneCurve *curve = cmsBuildGamma(p->cms, icc->gamma);
+    // Create approximation profile. Use a tone-curve based on a BT.1886-style
+    // pure power curve, with an approximation gamma matched to the ICC
+    // profile. We stretch the luminance range *before* the input to the gamma
+    // function, to avoid numerical issues near the black point. (This removes
+    // the need for a separate linear section)
+    //
+    // Y = scale * (aX + b)^y, where Y = PCS luma and X = encoded value ([0-1])
+    p->scale = pl_hdr_rescale(PL_HDR_NITS, PL_HDR_NORM, icc->csp.hdr.max_luma);
+    p->b = powf(icc->csp.hdr.min_luma / icc->csp.hdr.max_luma, 1.0f / icc->gamma);
+    p->a = (1 - p->b);
+    cmsToneCurve *curve = cmsBuildParametricToneCurve(p->cms, 2,
+            (double[3]) { icc->gamma, p->a, p->b });
     if (!curve)
         goto error;
 
@@ -360,6 +406,22 @@ pl_icc_object pl_icc_open(pl_log log, const struct pl_icc_profile *profile,
     if (!p->approx)
         goto error;
 
+    // We need to create an ICC V2 profile because ICC V4 perceptual profiles
+    // have normalized semantics, but we want colorimetric mapping with BPC
+    cmsSetHeaderRenderingIntent(p->approx, icc->params.intent);
+    cmsSetProfileVersion(p->approx, 2.2);
+
+    // Hash all parameters affecting the generated 3DLUT
+    p->lut_sig = icc->signature;
+    pl_hash_merge(&p->lut_sig, params->intent);
+    pl_hash_merge(&p->lut_sig, params->size_r);
+    pl_hash_merge(&p->lut_sig, params->size_g);
+    pl_hash_merge(&p->lut_sig, params->size_b);
+    pl_hash_merge(&p->lut_sig, params->force_bpc);
+    union { double d; uint64_t u; } v = { .d = icc->csp.hdr.max_luma };
+    pl_hash_merge(&p->lut_sig, v.u);
+    // min luma depends only on the max luma and profile
+
     return icc;
 
 error:
@@ -367,6 +429,15 @@ error:
     return NULL;
 }
 
+static inline bool cache_load(pl_icc_object icc, uint64_t sig,
+                              uint8_t *cache, size_t size)
+{
+    if (icc->params.cache_load)
+        return icc->params.cache_load(icc->params.cache_priv, sig, cache, size);
+
+    return false;
+}
+
 static void fill_lut(void *datap, const struct sh_lut_params *params, bool decode)
 {
     pl_icc_object icc = params->priv;
@@ -375,10 +446,18 @@ static void fill_lut(void *datap, const struct sh_lut_params *params, bool decod
     cmsHPROFILE srcp = decode ? p->profile : p->approx;
     cmsHPROFILE dstp = decode ? p->approx  : p->profile;
 
+    int s_r = params->width, s_g = params->height, s_b = params->depth;
+    size_t data_size = s_r * s_g * s_b * sizeof(uint16_t[4]);
+    if (cache_load(icc, params->signature, datap, data_size)) {
+        pl_info(log, "Using cached 3DLUT (0x%"PRIX64")", params->signature);
+        return;
+    }
+
     clock_t start = clock();
     cmsHTRANSFORM tf = cmsCreateTransformTHR(p->cms, srcp, TYPE_RGB_16,
                                              dstp, TYPE_RGBA_16,
                                              icc->params.intent,
+                                             cmsFLAGS_BLACKPOINTCOMPENSATION |
                                              cmsFLAGS_NOCACHE | cmsFLAGS_NOOPTIMIZE);
     if (!tf)
         return;
@@ -386,7 +465,6 @@ static void fill_lut(void *datap, const struct sh_lut_params *params, bool decod
     clock_t after_transform = clock();
     pl_log_cpu_time(log, start, after_transform, "creating ICC transform");
 
-    int s_r = params->width, s_g = params->height, s_b = params->depth;
     uint16_t *tmp = pl_alloc(NULL, s_r * 3 * sizeof(tmp[0]));
     for (int b = 0; b < s_b; b++) {
         for (int g = 0; g < s_g; g++) {
@@ -400,12 +478,34 @@ static void fill_lut(void *datap, const struct sh_lut_params *params, bool decod
             size_t offset = (b * s_g + g) * s_r * 4;
             uint16_t *data = ((uint16_t *) datap) + offset;
             cmsDoTransform(tf, tmp, data, s_r);
+
+            if (!icc->params.force_bpc)
+                continue;
+
+            // Fix the black point manually. Work-around for "improper"
+            // profiles, as black point compensation should already have
+            // taken care of this normally.
+            const uint16_t knee = 16u << 8;
+            if (tmp[0] >= knee || tmp[1] >= knee)
+                continue;
+            for (int r = 0; r < s_r; r++) {
+                uint16_t s = (2 * tmp[1] + tmp[2] + tmp[r * 3]) >> 2;
+                if (s >= knee)
+                    break;
+                for (int c = 0; c < 3; c++)
+                    data[r * 3 + c] = (s * data[r * 3 + c] + (knee - s) * s) >> 12;
+            }
         }
     }
 
     pl_log_cpu_time(log, after_transform, clock(), "generating ICC 3DLUT");
     cmsDeleteTransform(tf);
     pl_free(tmp);
+
+    if (icc->params.cache_save) {
+        icc->params.cache_save(icc->params.cache_priv, params->signature,
+                               datap, data_size);
+    }
 }
 
 static void fill_decode(void *datap, const struct sh_lut_params *params)
@@ -421,37 +521,38 @@ static void fill_encode(void *datap, const struct sh_lut_params *params)
 void pl_icc_decode(pl_shader sh, pl_icc_object icc, pl_shader_obj *lut_obj,
                    struct pl_color_space *out_csp)
 {
+    struct icc_priv *p = PL_PRIV(icc);
     if (!sh_require(sh, PL_SHADER_SIG_COLOR, 0, 0))
         return;
 
-    uint64_t sig = icc->signature;
-    pl_hash_merge(&sig, icc->params.intent);
-
     ident_t lut = sh_lut(sh, sh_lut_params(
         .object     = lut_obj,
-        .type       = PL_VAR_FLOAT,
+        .var_type   = PL_VAR_FLOAT,
+        .method     = SH_LUT_TETRAHEDRAL,
         .fmt        = pl_find_fmt(SH_GPU(sh), PL_FMT_UNORM, 4,
                                   16, 16, PL_FMT_CAP_LINEAR),
         .width      = icc->params.size_r,
         .height     = icc->params.size_g,
         .depth      = icc->params.size_b,
         .comps      = 4,
-        .linear     = true,
-        .signature  = sig,
+        .signature  = p->lut_sig,
         .fill       = fill_decode,
         .priv       = (void *) icc,
     ));
 
+    // Y = scale * (aX + b)^y
     sh_describe(sh, "ICC 3DLUT");
     GLSL("// pl_icc_decode                      \n"
          "{                                     \n"
          "color.rgb = %s(color.rgb).rgb;        \n"
-         "color.rgb = max(color.rgb, 0.0);      \n"
+         "color.rgb = %s * color.rgb + vec3(%s);\n"
          "color.rgb = pow(color.rgb, vec3(%s)); \n"
-         "color.rgb = %s * color.rgb;           \n"  // expand HDR levels
+         "color.rgb = %s * color.rgb;           \n"
          "}                                     \n",
-         lut, SH_FLOAT(icc->gamma),
-         SH_FLOAT(pl_hdr_rescale(PL_HDR_NITS, PL_HDR_NORM, icc->csp.hdr.max_luma)));
+         lut,
+         SH_FLOAT(p->a), SH_FLOAT(p->b),
+         SH_FLOAT(icc->gamma),
+         SH_FLOAT(p->scale));
 
     if (out_csp) {
         *out_csp = (struct pl_color_space) {
@@ -464,34 +565,42 @@ void pl_icc_decode(pl_shader sh, pl_icc_object icc, pl_shader_obj *lut_obj,
 
 void pl_icc_encode(pl_shader sh, pl_icc_object icc, pl_shader_obj *lut_obj)
 {
+    struct icc_priv *p = PL_PRIV(icc);
     if (!sh_require(sh, PL_SHADER_SIG_COLOR, 0, 0))
         return;
 
     ident_t lut = sh_lut(sh, sh_lut_params(
         .object     = lut_obj,
-        .type       = PL_VAR_FLOAT,
+        .var_type   = PL_VAR_FLOAT,
+        .method     = SH_LUT_TETRAHEDRAL,
         .fmt        = pl_find_fmt(SH_GPU(sh), PL_FMT_UNORM, 4,
                                   16, 16, PL_FMT_CAP_LINEAR),
         .width      = icc->params.size_r,
         .height     = icc->params.size_g,
         .depth      = icc->params.size_b,
         .comps      = 4,
-        .linear     = true,
-        .signature  = ~icc->signature, // avoid confusion with decoding LUTs
+        .signature  = ~p->lut_sig, // avoid confusion with decoding LUTs
         .fill       = fill_encode,
         .priv       = (void *) icc,
     ));
 
+    // X = 1/a * (Y/scale)^(1/y) - b/a
     sh_describe(sh, "ICC 3DLUT");
     GLSL("// pl_icc_encode                      \n"
          "{                                     \n"
-         "color.rgb = 1.0/%s * color.rgb;       \n"
+         "if (any(greaterThan(color.rgb, vec3(%s)))) \n"
+         "    color.rgb = vec3(1,0,0); \n"
          "color.rgb = max(color.rgb, 0.0);      \n"
+         "color.rgb = 1.0/%s * color.rgb;       \n"
          "color.rgb = pow(color.rgb, vec3(%s)); \n"
+         "color.rgb = 1.0/%s * color.rgb - %s;  \n"
          "color.rgb = %s(color.rgb).rgb;        \n"
          "}                                     \n",
-         SH_FLOAT(pl_hdr_rescale(PL_HDR_NITS, PL_HDR_NORM, icc->csp.hdr.max_luma)),
-         SH_FLOAT(1.0f / icc->gamma), lut);
+         SH_FLOAT(icc->csp.hdr.min_luma + 1e4),
+         SH_FLOAT(p->scale),
+         SH_FLOAT(1.0f / icc->gamma),
+         SH_FLOAT(p->a), SH_FLOAT(p->b / p->a),
+         lut);
 }
 
 #else // !PL_HAVE_LCMS
