@@ -176,9 +176,9 @@ static const VkShaderStageFlags stageFlags[] = {
     [PL_PASS_COMPUTE] = VK_SHADER_STAGE_COMPUTE_BIT,
 };
 
-static void destroy_pipeline(struct vk_ctx *vk, VkPipeline pipeline)
+static void destroy_pipeline(struct vk_ctx *vk, void *pipeline)
 {
-    vk->DestroyPipeline(vk->dev, pipeline, PL_VK_ALLOC);
+    vk->DestroyPipeline(vk->dev, vk_unwrap_handle(pipeline), PL_VK_ALLOC);
 }
 
 static VkResult vk_recreate_pipelines(struct vk_ctx *vk, pl_pass pass,
@@ -191,8 +191,8 @@ static VkResult vk_recreate_pipelines(struct vk_ctx *vk, pl_pass pass,
     // The old pipeline might still be in use, so we have to destroy it
     // asynchronously with a device idle callback
     if (*out_pipe) {
-        vk_dev_callback(vk, (vk_cb) destroy_pipeline, vk, *out_pipe);
-        *out_pipe = NULL;
+        vk_dev_callback(vk, (vk_cb) destroy_pipeline, vk, vk_wrap_handle(*out_pipe));
+        *out_pipe = VK_NULL_HANDLE;
     }
 
     VkPipelineCreateFlags flags = 0;
@@ -342,7 +342,7 @@ pl_pass vk_pass_create(pl_gpu gpu, const struct pl_pass_params *params)
     struct vk_ctx *vk = p->vk;
     bool success = false;
 
-    struct pl_pass *pass = pl_zalloc_obj(NULL, pass, struct pl_pass_vk);
+    struct pl_pass_t *pass = pl_zalloc_obj(NULL, pass, struct pl_pass_vk);
     pass->params = pl_pass_params_copy(pass, params);
 
     struct pl_pass_vk *pass_vk = PL_PRIV(pass);
@@ -612,7 +612,7 @@ no_descriptors: ;
 
     // Create the graphics/compute pipeline
     VkPipeline *pipe = has_spec ? &pass_vk->base : &pass_vk->pipe;
-    VK(vk_recreate_pipelines(vk, pass, has_spec, NULL, pipe));
+    VK(vk_recreate_pipelines(vk, pass, has_spec, VK_NULL_HANDLE, pipe));
     pl_log_cpu_time(gpu->log, after_compilation, clock(), "creating pipeline");
 
     if (!has_spec) {
