@@ -22,7 +22,6 @@
 
 #ifdef PL_HAVE_UNIX
 #include <errno.h>
-#include <strings.h>
 #include <unistd.h>
 #endif
 
@@ -238,6 +237,9 @@ static void slab_free(struct vk_ctx *vk, struct vk_slab *slab)
             PL_TRACE(vk, "Unimporting slab of size %s from ptr: %p",
                      PRINT_SIZE(slab->size), (void *) slab->handle.ptr);
             break;
+        case PL_HANDLE_IOSURFACE:
+        case PL_HANDLE_MTL_TEX:
+            pl_unreachable();
         }
     } else {
         switch (slab->handle_type) {
@@ -260,6 +262,9 @@ static void slab_free(struct vk_ctx *vk, struct vk_slab *slab)
         case PL_HANDLE_HOST_PTR:
             // Implicitly unmapped
             break;
+        case PL_HANDLE_IOSURFACE:
+        case PL_HANDLE_MTL_TEX:
+            pl_unreachable();
         }
 
         PL_DEBUG(vk, "Freeing slab of size %s", PRINT_SIZE(slab->size));
@@ -338,7 +343,9 @@ static bool buf_external_check(struct vk_ctx *vk, VkBufferUsageFlags usage,
         .sType = VK_STRUCTURE_TYPE_EXTERNAL_BUFFER_PROPERTIES_KHR,
     };
 
-    pl_assert(info.handleType);
+    if (!info.handleType)
+        return false;
+
     vk->GetPhysicalDeviceExternalBufferProperties(vk->physd, &info, &props);
     return vk_external_mem_check(vk, &props.externalMemoryProperties,
                                  handle_type, import);
@@ -364,6 +371,8 @@ static struct vk_slab *slab_alloc(struct vk_malloc *ma,
         break;
     case PL_HANDLE_WIN32:
     case PL_HANDLE_WIN32_KMT:
+    case PL_HANDLE_MTL_TEX:
+    case PL_HANDLE_IOSURFACE:
         slab->handle.handle = NULL;
         break;
     case PL_HANDLE_HOST_PTR:
@@ -841,6 +850,8 @@ static bool vk_malloc_import(struct vk_malloc *ma, struct vk_memslice *out,
     case PL_HANDLE_FD:
     case PL_HANDLE_WIN32:
     case PL_HANDLE_WIN32_KMT:
+    case PL_HANDLE_IOSURFACE:
+    case PL_HANDLE_MTL_TEX:
         PL_ERR(vk, "vk_malloc_import: unsupported handle type %d",
                params->import_handle);
         goto error;
@@ -853,7 +864,7 @@ static bool vk_malloc_import(struct vk_malloc *ma, struct vk_memslice *out,
         goto error;
     }
 
-    VkDeviceMemory vkmem = NULL;
+    VkDeviceMemory vkmem = VK_NULL_HANDLE;
     VK(vk->AllocateMemory(vk->dev, &ainfo, PL_VK_ALLOC, &vkmem));
 
     slab = pl_alloc_ptr(NULL, slab);
@@ -894,6 +905,8 @@ static bool vk_malloc_import(struct vk_malloc *ma, struct vk_memslice *out,
         break;
     case PL_HANDLE_WIN32:
     case PL_HANDLE_WIN32_KMT:
+    case PL_HANDLE_IOSURFACE:
+    case PL_HANDLE_MTL_TEX:
         break;
     }
 
